 PROGRAM CALOR 
 IMPLICIT NONE

 REAL, PARAMETER        :: XI               = 0.0
 REAL, PARAMETER        :: XF               = 1.0
 REAL, PARAMETER        :: TEMPO_FINAL      = 10.0
 REAL, PARAMETER        :: ALFA             = 1.0
 REAL, PARAMETER        :: TOL_SOL          = 1.0E-04 ! TOLERÂNCIA NA SOLUÇÃO NUMÉRICA DA EQ. DO CALOR
 REAL, PARAMETER        :: TOL_PERMANENTE   = 5.0E-03 ! TOLERÂNCIA PARA ALCANÇAR REGIME PERMANENTE
 REAL, PARAMETER        :: W_SOL            = 1.0	  ! GAUSS-SEIDEL
 REAL, PARAMETER        :: DELTA_T          = 1.0E-03 ! TAMANHO DO PASSO DE TEMPO
 INTEGER, PARAMETER     :: N                = 50      ! NÚMERO DE PONTOS 
 INTEGER, PARAMETER     :: ITMAX_SOL        = 1E+06   ! NÚMERO MÁX. DE ITERAÇÕES PARA RESOLVER A EQ. DO CALOR
 INTEGER, PARAMETER     :: AMOSTRA          = 10      ! VALOR QUE DEFINE O TAMANHO DA AMOSTRA DE SAÍDAS 

 REAL                           :: AE, AP, AW,B,MAIOR_DIFERENCA1, MAXIMO1,MAXIMO2 
 REAL                           :: TEMPO, UNOVO,MAIOR_DIFERENCA2, DELTA_X
 REAL, DIMENSION(:),ALLOCATABLE :: X,U,UVELHO
 INTEGER                        :: I,IT,GRAFICO,MANIPULA,CONTADOR 

 
 
 ! SE A PASTA SAIDAS EXISTE ENTÃO ELA É DELETADA
 MANIPULA = SYSTEM('if test -e SAIDAS; then rm -r SAIDAS; fi;') 
 
 ! CRIA A PASTA DAS SAÍDAS
 MANIPULA = SYSTEM('mkdir ./'//TRIM('SAIDAS'))

 
 OPEN (3, FILE='DADOS_SIMULACAO_TRANSIENTE.dat', STATUS='UNKNOWN')

 WRITE(3,*)'TEMPO','              ERRO - TRANSIENTE'



 ALLOCATE(X(N),U(N),UVELHO(N))

 
 
 ! DEFINIR A MALHA

 DELTA_X = (XF-XI)/(N-1)

 DO I=1, N
    X(I) = XI + (I-1)*DELTA_X;
 END DO


 CONTADOR = 0

 GRAFICO  = 0
 
 ! CONDIÇÃO DE CONTORNO

 U(1) = 0.0
 U(N) = 0.0


 !  CONDIÇÃO INICIAL 

 TEMPO = 0.0

 DO I= 2, N-1
    U(I) = 1.0
 END DO

 
 CALL GRAVACAO_SAIDAS(GRAFICO,N,X,U)
 
 
 
 !CALCULO DOS COEFICIENTES DA EQUACAO DISCRETIZADA IMPLICITO

 AW = ((ALFA**2.0)*DELTA_T) / (DELTA_X**2.0)
 AE = AW
 AP = 1.0 + 2.0 * ( ((ALFA**2.0)*DELTA_T) / (DELTA_X**2.0) )
 B  = 1.0


 
 
 DO WHILE (TEMPO < TEMPO_FINAL) 

	CONTADOR = CONTADOR + 1
	
	TEMPO = TEMPO + DELTA_T
    
	MAIOR_DIFERENCA2 = 0.0 
	
    	DO I=1,N
        	UVELHO(I)= U(I) ! UVELHO É O VALOR DE U NO TEMPO ANTERIOR	
    	END DO

    	DO IT=1,ITMAX_SOL 
        	
        	MAIOR_DIFERENCA1 = 0.0
       		
       		DO I=2,N-1
                		    
                ! CALCULO DA EQUACAO DISCRETIZADA ATRAVES DE RELAXACOES SUCESSIVAS

                UNOVO = (1.0/AP) * ( B*UVELHO(I) + AE*U(I+1) + AW*U(I-1) )

                MAXIMO1 = ABS(UNOVO-U(I))/ABS(UNOVO) 
		    
                IF (MAXIMO1 > MAIOR_DIFERENCA1) THEN ! CALCULO DO ERRO
                    MAIOR_DIFERENCA1 = MAXIMO1
                END IF
		  
                MAXIMO2 = ABS(UNOVO-UVELHO(I))/ABS(UNOVO)
		    
                IF (MAXIMO2 > MAIOR_DIFERENCA2) THEN
                    MAIOR_DIFERENCA2 = MAXIMO2
                END IF
		  
                U(I) = UNOVO
		    
            END DO  
                        
            IF (MAIOR_DIFERENCA1 <= TOL_SOL) THEN
                GOTO 113
            END IF
                
            IF (IT==ITMAX_SOL) THEN
                WRITE(*,11)IT
                11  FORMAT(///,10X,'SOLUCAO """ NAO """ OBTIDA APOS ',I10,' ITERACOES',//)
                STOP                                          
            END IF
    	                
        END DO

        113 CONTINUE
                        
        WRITE(3,10)TEMPO,MAIOR_DIFERENCA2
        10  FORMAT(F10.4,10X,ES12.6)
                    
        IF(MOD(CONTADOR,AMOSTRA)==0) THEN
            GRAFICO = GRAFICO + 1
            CALL GRAVACAO_SAIDAS(GRAFICO,N,X,U)
        END IF
        
        
        IF (MAIOR_DIFERENCA2 <= TOL_PERMANENTE) THEN
            WRITE(3,*)TEMPO,MAIOR_DIFERENCA2
            WRITE(3,12)TEMPO
            12  FORMAT(10X,' SOLUCAO EM REGIME PERMANENTE OBTIDA APOS ',F20.8,' PASSOS DE TEMPO',/)
            STOP
        END IF
        
!         TEMPO = TEMPO + DELTA_T
        
 END DO
	    
 END PROGRAM CALOR
